import { JSDOM } from 'jsdom';
import axios from 'axios';
import puppeteer from 'puppeteer';

var EngineType;
(function (EngineType) {
    EngineType["Html"] = "html";
    EngineType["Spa"] = "spa";
    EngineType["Json"] = "json";
    EngineType["Xml"] = "xml";
})(EngineType || (EngineType = {}));
var PluginType;
(function (PluginType) {
    PluginType["Transformer"] = "transformer";
    PluginType["Engine"] = "engine";
})(PluginType || (PluginType = {}));

const DEFAULT_HTTP_REQUESTS_OPTIONS = {
    headers: {
        "Cache-Control": "no-cache",
        "Accept-Encoding": "gzip, deflate",
    },
};

const request$1 = async (url, request = {}) => {
    try {
        const response = await axios.get(url, {
            ...DEFAULT_HTTP_REQUESTS_OPTIONS,
            ...request,
        });
        return response.data;
    }
    catch (error) {
        throw new Error(`Error fetching data from ${url}: ${error.message}`);
    }
};

const getSelectorValue = (element, selector) => {
    const value = selector.attribute
        ? element.getAttribute(selector.attribute) || ""
        : element.textContent?.trim() || "";
    return value;
};
const extractData = (document, selectors) => {
    const data = {};
    for (const [key, selector] of Object.entries(selectors)) {
        const elements = document.querySelectorAll(selector.selector);
        if (!elements.length) {
            data[key] = "";
        }
        else if (elements.length === 1) {
            data[key] = getSelectorValue(elements[0], selector) || "";
        }
        else {
            data[key] = Array.from(elements)
                .map((element) => getSelectorValue(element, selector))
                .filter((value) => value !== null);
        }
    }
    return data;
};

const processData$3 = async (url, options) => {
    const { selectors, request, plugins } = options;
    let html = await request$1(url, request);
    plugins?.forEach((plugin) => {
        if (plugin.onPreProcess) {
            html = plugin.onPreProcess(html);
        }
    });
    const dom = new JSDOM(html);
    const document = dom.window.document;
    return extractData(document, selectors);
};
const scrape$4 = async (url, { selectors, request, plugins = [] }) => {
    plugins?.forEach((plugin) => {
        if (plugin.type === PluginType.Transformer &&
            plugin.onInit &&
            plugin.supportedEngines?.includes(EngineType.Html || "all")) {
            plugin.onInit({ selectors });
        }
    });
    let data = await processData$3(url, { selectors, request, plugins });
    plugins?.forEach((plugin) => {
        if (plugin.onPostProcess) {
            data = plugin.onPostProcess(data);
        }
    });
    return data;
};
const plugin$4 = {
    type: "engine",
    engine: "html",
    scrape: scrape$4,
};

const autoScroll = async (page) => {
    await page.evaluate(async () => {
        await new Promise((resolve) => {
            let totalHeight = 0;
            let distance = 100;
            let timer = setInterval(() => {
                let scrollHeight = document.body.scrollHeight;
                window.scrollBy(0, distance);
                totalHeight += distance;
                if (totalHeight >= scrollHeight) {
                    clearInterval(timer);
                    resolve(true);
                }
            }, 100);
        });
    });
};
const request = async (url, config = {}) => {
    const browser = await puppeteer.launch({
        executablePath: config?.executablePath
            ? config.executablePath
            : puppeteer.executablePath(),
        ...config,
    });
    const page = await browser.newPage();
    await page.goto(url);
    await autoScroll(page);
    const content = await page.content();
    await browser.close();
    return content;
};

const processData$2 = async (url, options) => {
    const { selectors, request: request$1, plugins } = options;
    let html = await request(url, request$1);
    plugins?.forEach((plugin) => {
        if (plugin.onPreProcess) {
            html = plugin.onPreProcess(html);
        }
    });
    const dom = new JSDOM(html);
    const document = dom.window.document;
    return extractData(document, selectors);
};
const scrape$3 = async (url, { selectors, request, plugins }) => {
    plugins?.forEach((plugin) => {
        if (plugin.type === PluginType.Transformer &&
            plugin.onInit &&
            plugin?.supportedEngines?.includes(EngineType.Spa || "all")) {
            plugin.onInit({ selectors });
        }
    });
    let data = await processData$2(url, { selectors, request, plugins });
    plugins?.forEach((plugin) => {
        if (plugin.onPostProcess) {
            data = plugin.onPostProcess(data);
        }
    });
    return data;
};
const plugin$3 = {
    type: "engine",
    engine: "spa",
    scrape: scrape$3,
};

const processData$1 = async (url, { selectors, request, plugins }) => {
    let j = await request$1(url, request);
    plugins?.forEach((plugin) => {
        if (plugin.onPreProcess) {
            j = plugin.onPreProcess(j);
        }
    });
    return extractData(j, selectors);
};
const scrape$2 = async (url, { selectors, request, plugins }) => {
    plugins?.forEach((plugin) => {
        if (plugin.type === PluginType.Transformer &&
            plugin.onInit &&
            plugin?.supportedEngines?.includes(EngineType.Json || "all")) {
            plugin.onInit({ selectors });
        }
    });
    let data = await processData$1(url, { selectors, request, plugins });
    plugins?.forEach((plugin) => {
        if (plugin.onPostProcess) {
            data = plugin.onPostProcess(data);
        }
    });
    return data;
};
const plugin$2 = {
    type: "engine",
    engine: "json",
    scrape: scrape$2,
};

const processData = async (url, { selectors, request, plugins }) => {
    let xml = await request$1(url, request);
    plugins?.forEach((plugin) => {
        if (plugin.onPreProcess) {
            xml = plugin.onPreProcess(xml);
        }
    });
    const dom = new JSDOM(xml, { contentType: "text/xml" });
    const document = dom.window.document;
    return extractData(document, selectors);
};
const scrape$1 = async (url, { selectors, request, plugins }) => {
    plugins?.forEach((plugin) => {
        if (plugin.type === PluginType.Transformer &&
            plugin.onInit &&
            plugin?.supportedEngines?.includes(EngineType.Xml || "all")) {
            plugin.onInit({ selectors });
        }
    });
    let data = await processData(url, { selectors, request, plugins });
    plugins?.forEach((plugin) => {
        if (plugin.onPostProcess) {
            data = plugin.onPostProcess(data);
        }
    });
    return data;
};
const plugin$1 = {
    type: "engine",
    engine: "xml",
    scrape: scrape$1,
};

const operators = {
    difference: (a, b, sensitive) => sensitive ? a !== b : a.toLowerCase() !== b.toLowerCase(),
    equal: (a, b, sensitive) => sensitive ? a === b : a.toLowerCase() === b.toLowerCase(),
    contains: (a, b, sensitive) => sensitive ? a.includes(b) : a.toLowerCase().includes(b.toLowerCase()),
    regex: (a, b, sensitive) => new RegExp(b, sensitive ? "" : "i").test(a),
};
const applyConditions = (value, conditions) => {
    return conditions.every((condition) => {
        const operation = operators[condition.operation];
        return operation
            ? operation(value, condition.value, condition.sensitive || false)
            : false;
    });
};

const plugin = (conditions) => ({
    type: "transformer",
    supportedEngines: ["all"],
    onPostProcess(data) {
        const newData = {};
        for (const [key, value] of Object.entries(data)) {
            if (conditions[key]) {
                if (Array.isArray(value)) {
                    newData[key] = value.filter((v) => applyConditions(v, conditions[key]));
                    newData[key] =
                        newData[key]?.length === 1 ? newData[key][0] : newData[key];
                }
                else {
                    newData[key] = applyConditions(value, conditions[key]) ? value : null;
                }
            }
            else {
                newData[key] = value;
            }
        }
        return newData;
    },
});

var ConditionOperationType;
(function (ConditionOperationType) {
    ConditionOperationType["Difference"] = "difference";
    ConditionOperationType["Equal"] = "equal";
    ConditionOperationType["Contains"] = "contains";
    ConditionOperationType["Regex"] = "regex";
})(ConditionOperationType || (ConditionOperationType = {}));

const SCRAPE_TYPES = {
    html: plugin$4.scrape,
    spa: plugin$3.scrape,
    json: plugin$2.scrape,
    xml: plugin$1.scrape,
};
const registerEnginePlugin = (plugin) => {
    SCRAPE_TYPES[plugin.engine] = plugin.scrape;
};

const isEngineTypeRegistered = (engine) => {
    return engine in SCRAPE_TYPES;
};
const scrape = async ({ url, engine, options, }) => {
    if (!isEngineTypeRegistered(engine)) {
        throw new Error(`Invalid engine type: ${engine}`);
    }
    return await SCRAPE_TYPES[engine](url, options);
};

export { ConditionOperationType, EngineType, PluginType, plugin as conditions, registerEnginePlugin, scrape, scrape$4 as scrapeHtml, scrape$2 as scrapeJson, scrape$3 as scrapeSpa, scrape$1 as scrapeXml };
//# sourceMappingURL=index.js.map
